@@ -1,5 +1,4 @@
-unique.id <-
-function(x)
+unique.id <- function(x)
 {
   rval <- .Call("unique_id",
     as.numeric(x),
