@@ -131,7 +131,7 @@ write.bayesx.input <- function(object)
           ff <- as.data.frame(eval(parse(text = paste("model.matrix(~ -1 +", sf,")", 
             sep = "")), envir = object$data))
         } else {
-          ff <- as.data.frame(eval(parse(text = paste("model.matrix(~ 1", sf,")", 
+          ff <- as.data.frame(eval(parse(text = paste("model.matrix(~ 1 +", sf,")", 
             sep = "")), envir = object$data))
         }
         lf <- colnames(ff)
@@ -172,6 +172,11 @@ write.bayesx.input <- function(object)
       }
     }
     data.file <- paste(object$outfile, "/", object$model.name, ".data.raw", sep = "")
+    if(!is.null(object$begin.vec)) {
+      dn <- c(colnames(dat), object$begin)
+      dat <- cbind(dat, object$begin.vec)
+      colnames(dat) <- dn
+    }
     if(!file.exists(data.file))
       write.table(dat, data.file, col.names = TRUE, row.names = FALSE, quote = FALSE)
     else {
